@@ -37,6 +37,13 @@
 
     @see <https://libguestfs.org/nbdkit-plugin.3.html> nbdkit-plugin(3) *)
 
+(** {2 Exception that plugins can raise}
+
+    Optionally plugins can raise this exception to control how
+    nbdkit_set_error and nbdkit_error are called. *)
+
+exception Error of Unix.error option * string
+
 (** {2 Flags passed from the server to various callbacks} *)
 
 type flags = flag list
@@ -82,6 +89,24 @@ type buf =
 val buf_len : buf -> int
 (** Return the length of the [~pread] and [~pwrite] {!buf} parameter. *)
 
+val buf_create : int -> buf
+(** Create {!buf} of the given length.  The content is undefined. *)
+
+val buf_zeroes : int -> buf
+(** Create {!buf} of the given length containing all zero bytes. *)
+
+val set_buf_to_zero : buf -> unit
+(** Set all bytes in an existing buffer to zero. *)
+
+val buf_of_string : string -> buf
+val string_of_buf : buf -> string
+val buf_of_bytes : bytes -> buf
+val bytes_of_buf : buf -> bytes
+(** Convenience functions to convert to and from string/bytes and buf.
+
+    These are not very efficient.  Use {!module:Bigarray.Array1} functions
+    instead for efficiency. *)
+
 val blit_string_to_buf : string -> int -> buf -> int -> int -> unit
 (** [blit_string_to_buf src src_pos buf buf_pos len] copies
     [len] bytes from string [src+src_pos] to [buf+buf_pos] *)
@@ -174,6 +199,17 @@ val debug : ('a, unit, string, unit) format4 -> 'a
 
     @see <https://libguestfs.org/nbdkit_debug.3.html> nbdkit_debug(3) *)
 
+val debug_hexdump : buf -> string option -> int64 -> unit
+(** Display buffer in canonical hexdump format, sending the result
+    to debug output (when in verbose mode).
+
+    @see <https://libguestfs.org/nbdkit_debug_hexdump.3.html> nbdkit_debug_hexdump(3) *)
+
+val debug_hexdiff : buf -> buf -> string option -> int64 -> unit
+(** Display differences in two buffers (of equal length).
+
+    @see <https://libguestfs.org/nbdkit_debug_hexdiff.3.html> nbdkit_debug_hexdiff(3) *)
+
 (** {2 Parsing and configuration} *)
 
 (* Note OCaml has functions already for parsing other integers, so
