@@ -34,6 +34,7 @@
 
 #include <stdio.h>
 #include <stdlib.h>
+#include <stdbool.h>
 #include <string.h>
 #include <assert.h>
 #include <unistd.h>
@@ -48,6 +49,7 @@
 #include <caml/mlvalues.h>
 #include <caml/printexc.h>
 #include <caml/threads.h>
+#include <caml/unixsupport.h>
 #include <caml/version.h>
 
 #define NBDKIT_API_VERSION 2
@@ -215,17 +217,60 @@ plugin_init (void)
 /*----------------------------------------------------------------------*/
 /* Wrapper functions that translate calls from C (ie. nbdkit) to OCaml. */
 
+static bool
+exception_to_error (const char *func, value rv)
+{
+  CAMLparam0 ();
+  CAMLlocal2 (exn, v);
+  const bool is_exn = Is_exception_result (rv);
+  const char *exn_name;
+
+  if (is_exn) {
+    /* See libguestfs.git daemon/daemon-c.c */
+    exn = Extract_exception (rv);
+
+    if (Tag_val (Field (exn, 0)) == String_tag)
+      exn_name = String_val (Field (exn, 0));
+    else
+      exn_name = String_val (Field (Field (exn, 0), 0));
+
+    nbdkit_debug ("ocaml: '%s' raised exception '%s'", func, exn_name);
+
+    if (strcmp (exn_name, "NBDKit.Error") == 0) {
+      v = Field (exn, 1);
+      if (v != Val_int (0)) /* Some error */
+        nbdkit_set_error (code_of_unix_error (Field (v, 0)));
+      nbdkit_error ("%s", String_val (Field (exn, 2)));
+    }
+    else if (strcmp (exn_name, "Failure") == 0) {
+      nbdkit_error ("%s", String_val (Field (exn, 1)));
+    }
+    else if (strcmp (exn_name, "Invalid_argument") == 0) {
+      nbdkit_error ("invalid argument: %s", String_val (Field (exn, 1)));
+    }
+    else if (strcmp (exn_name, "Unix.Unix_error") == 0) {
+      nbdkit_set_error (code_of_unix_error (Field (exn, 1)));
+      nbdkit_error ("%s: %s: %s", func,
+                    String_val (Field (exn, 2)),
+                    String_val (Field (exn, 3)));
+    }
+    else {
+      /* General exception. */
+      nbdkit_error ("%s: %s", func, caml_format_exception (exn));
+    }
+  }
+
+  CAMLreturnT (bool, is_exn);
+}
+
 /* This macro calls nbdkit_error when we get an exception thrown in
  * OCaml callback code.  The 'return_stmt' parameter is usually a call
  * to CAMLreturnT, but may be empty in order to fall through.
  */
 #define EXCEPTION_TO_ERROR(rv, return_stmt)                             \
   do {                                                                  \
-    if (Is_exception_result (rv)) {                                     \
-      nbdkit_error ("%s: %s", __func__,                                 \
-                    caml_format_exception (Extract_exception (rv)));    \
+    if (exception_to_error (__func__, (rv)))                            \
       return_stmt;                                                      \
-    }                                                                   \
   } while (0)
 
 static void
